import json
from tqdm import tqdm

def count_overweight(file_name):
    with open(file_name,'rb') as input_file:
        records = json.load(input_file)
    counter = 0
    for i in tqdm(records):
        if i.get("BmiCategory") == "Over Weight":
            counter += 1
    print("No of overweight people in records is :",counter)
    


import json
import bisect
from typing import List
from tqdm import tqdm
from .constants import bmi_scale,bmi_category,catch_exceptions

class BodyMassIndex:
    """
    Class contains functions to calculate,categorise BMI.
    """
    def __init__(self) -> None:
        pass

    @catch_exceptions
    def convert_cm_to_m(self,height : float) -> float:
        """
        This function takes height in CMs and returns heights in meters
        """
        return height/100.0 

    @catch_exceptions
    def calculate_bmi(self,record : json) -> json:
        """
        record : {
                    "Gender":"Male",
                    "HeightCm":171,
                    "WeightKg":96
            }
        
        BMI = Weight / (Height ** 2) (Kgs/m^2)
        """
        height = self.convert_cm_to_m(record.get("HeightCm"))
        weight = record.get("WeightKg")
        BMI = round(weight / (height**2),2)
        
        #bisect_left gives the left most index where the calculated BMI falls on the scale, this index is inturn used to retrieve BMI category.
        #0.01 is deliberatly added while getting index inorder to get correct index on edge cases.
        bmi_info = bmi_category.get(bisect.bisect_left(bmi_scale,(BMI+0.01))-1) 
        
        bmi_info["BMI"] = BMI
        return bmi_info

    @catch_exceptions
    def process_data(self,records : List) -> List:
        """
        records : [
                {
                    "Gender":"Male",
                    "HeightCm":171,
                    "WeightKg":96
                },
                {
                    "Gender":"Male",
                    "HeightCm":161,
                    "WeightKg":85
                },
        ]
        Takes records and processes and returns updated records.
        """
        for i in tqdm(records):
            i.update(self.calculate_bmi(i))
        return records

    @catch_exceptions
    def process_start(self,input_file_path : str ,output_file_path : str):
        """
        input_file_path : input json file path
        output_file_path : path to dump the processed data.
        """
        if input_file_path == None or output_file_path  == None:
            raise Exception("Input file or Output file paths does not exist")
        
        with open(input_file_path,'rb') as input_file:
            input_records = json.load(input_file)

        processed_data = self.process_data(input_records)

        with open(output_file_path,'w') as outfile:
            json.dump(processed_data,outfile,indent=4)

BMI = BodyMassIndex()


# import bisect
import time
import logging

def catch_exceptions(func):
    def wrapper(*args, **kwargs):
        try:
            start_time = time.time()
            return_value = func(*args, **kwargs)
            end_time = time.time()
            # logger = logging.getLogger(func.__name__)
            # logger.warning("Function {} took {} seconds to complete\n".format(func.__name__,end_time-start_time))

            return return_value
        except Exception as e:
            logger = logging.getLogger(func.__name__)
            # print((args),"***************")
            logger.error("Error in {} :".format(func.__name__) + str(e),exc_info=False)

            return e
    return wrapper

bmi_scale =  [0,18.4,  18.5,24.9,  25,29.9,  30,34.9,  35,39.9,  40,10000]

bmi_category = {
    0:{
        "BmiCategory":"Under Weight",
        "HealthRisk":"Malnutrition Risk"
    },
    2:{
        "BmiCategory":"Normal Weight",
        "HealthRisk":"Low Risk"
    },
    4:{
        "BmiCategory":"Over Weight",
        "HealthRisk":"Enhanced Risk"
    },
    6:{
        "BmiCategory":"Moderately Obese",
        "HealthRisk":"Medium Risk"
    },
    8:{
        "BmiCategory":"Severely Obese",
        "HealthRisk":"High Risk"
    },
    10:{
        "BmiCategory":"Very Severely Obese",
        "HealthRisk":"Very High Risk"
    }
}


# print(bisect.bisect_left(bmi_scale,16)-1)
# print(bisect.bisect_left(bmi_scale,20.01)-1)
# print(bisect.bisect_left(bmi_scale,24.8)-1)
# print(bisect.bisect_left(bmi_scale,28)-1)
# print(bisect.bisect_left(bmi_scale,30.01)-1)
# print(bisect.bisect_left(bmi_scale,32)-1)
# print(bisect.bisect_left(bmi_scale,38)-1)
# print(bisect.bisect_left(bmi_scale,40.01)-1)
# print(bisect.bisect_left(bmi_scale,700)-1)
# print(bisect.bisect_right(bmi_scale,35))


from setuptools import setup

setup(
    name='bmi_batch_process',
    url='https://github.com/jagadeesh-r1/code-20210626-jagadeeshreddy',
    author='Jagadeesh Reddy',
    author_email='mail4jagadeeshreddy@gmail.com',
    packages=['BMI'],
    version='0.3',
    license='GNU General Public License v3.0',
    description='This package is used to process BMI (BodyMassIndex) using Height and Weight of person. The input must a iterable of json objects',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown'
)
